@@ -92,6 +92,7 @@ class OEMHandler(generic.OEMHandler):
         super(OEMHandler, self).__init__(sysinfo, sysurl, webclient, cache,
                                          gpool)
         self._wc = None
+        self.weblogging = False
         self.updating = False
         self.datacache = {}
 
@@ -567,6 +568,9 @@ class OEMHandler(generic.OEMHandler):
         url = '/api/function/raid_conf?params=raidlink_GetDefaultVolProp'
         args = (url, cid, 0, params['drives'])
         props = self.wc.grab_json_response(','.join([str(x) for x in args]))
+        if not props:  # newer firmwarerequires raidlevel too
+            args = (url, cid, params['raidlevel'], 0, params['drives'])
+            props = self.wc.grab_json_response(','.join([str(x) for x in args]))
         props = props['items'][0]
         volumes = pool.volumes
         remainingcap = params['capacity']
@@ -638,14 +642,21 @@ class OEMHandler(generic.OEMHandler):
         parms = {'raidlink_AddNewVolWithNaAsync': arglist}
         rsp = self.wc.grab_json_response(url, parms)
         if rsp['return'] == 14:  # newer firmware
-            if cid[2] == 2:
-                cnum = cid[1]
-            arglist = '{0},{1},{2},{3},{4},{5},'.format(
+            arglist = '{0},{1},{2},{3},{4},{5},{6},'.format(
                 cnum, params['raidlevel'], params['spans'],
-                params['perspan'], params['drives'], params['hotspares'])
-            arglist += ''.join(vols) + ',{0}'.format(cid[2])
+                params['perspan'], 0, params['drives'], params['hotspares'])
+            arglist += ''.join(vols)
             parms = {'raidlink_AddNewVolWithNaAsync': arglist}
             rsp = self.wc.grab_json_response(url, parms)
+            if not rsp: # Purley
+                if cid[2] == 2:
+                    cnum = cid[1]
+                arglist = '{0},{1},{2},{3},{4},{5},'.format(
+                    cnum, params['raidlevel'], params['spans'],
+                    params['perspan'], params['drives'], params['hotspares'])
+                arglist += ''.join(vols) + ',{0}'.format(cid[2])
+                parms = {'raidlink_AddNewVolWithNaAsync': arglist}
+                rsp = self.wc.grab_json_response(url, parms)
         if rsp['return'] != 0:
             raise Exception(
                 'Unexpected response to add volume command: ' + repr(rsp))
@@ -688,12 +699,30 @@ class OEMHandler(generic.OEMHandler):
             if pool.disks:
                 self._create_array(pool)
 
+    def weblogout(self):
+        if self._wc:
+            try:
+                self._wc.grab_json_response(self.logouturl)
+            except Exception:
+                pass
+            self._wc = None
+
     @property
     def wc(self):
-        if (not self._wc or (self._wc.vintage
-                             and self._wc.vintage < util._monotonic_time()
-                             - 30)):
-            self._wc = self.get_webclient()
+        while self.weblogging:
+            time.sleep(0.25)
+        self.weblogging = True
+        try:
+            if (not self._wc or (self._wc.vintage
+                                and self._wc.vintage < util._monotonic_time()
+                                - 30)):
+                if not self.updating and self._wc:
+                    # in case the existing session is still valid
+                    # dispose of the session
+                    self.weblogout()
+                self._wc = self.get_webclient()
+        finally:
+            self.weblogging = False
         return self._wc
 
     def get_webclient(self, login=True):
